/*
 * userlist.h - high-level roster
 * Copyright (C) 2001, 2002  Justin Karneges
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef USERLIST_H
#define USERLIST_H

#include<qstring.h>
#include<qdatetime.h>
#include<qvaluelist.h>
#include"kmp/jid.h"
#include"kmp/client.h"


class UserResource : public Jabber::Resource
{
public:
	UserResource();
	UserResource(const Jabber::Resource &);
	~UserResource();

	void setResource(const Jabber::Resource &);

	const QString & versionString() const;
	void setVersionString(const QString &);

private:
	QString v_ver;
};

bool operator<(const UserResource &r1, const UserResource &r2);
bool operator<=(const UserResource &r1, const UserResource &r2);
bool operator==(const UserResource &r1, const UserResource &r2);
bool operator>(const UserResource &r1, const UserResource &r2);
bool operator>=(const UserResource &r1, const UserResource &r2);

class UserResourceList : public QValueList<UserResource>
{
public:
	UserResourceList();
	~UserResourceList();

	void sort();

	UserResourceList::Iterator find(const QString &);
	UserResourceList::Iterator priority();

	UserResourceList::ConstIterator find(const QString &) const;
	UserResourceList::ConstIterator priority() const;
};

class UserListItem : public Jabber::LiveRosterItem
{
public:
	UserListItem(bool self=false);
	~UserListItem();

	bool inList() const;
	bool isTransport() const;
	bool isAvailable() const;
	bool isAway() const;
	QDateTime lastAvailable() const;
	const QString & presenceError() const;
	bool isSelf() const;
	QString makeTip(bool trim=true) const;
	QString makeDesc() const;

	void setJid(const Jabber::Jid &);
	void setInList(bool);
	void setLastAvailable(const QDateTime &);
	void setPresenceError(const QString &);

	UserResourceList & userResourceList();
	UserResourceList::Iterator priority();
	const UserResourceList & userResourceList() const;
	UserResourceList::ConstIterator priority() const;

private:
	bool v_inList;
	QDateTime v_t;
	UserResourceList v_url;
	QString v_perr;
	bool v_self, v_isTransport;
};

typedef QPtrListIterator<UserListItem> UserListIt;

class UserList : public QPtrList<UserListItem>
{
public:
	UserList();
	~UserList();

	UserListItem *find(const Jabber::Jid &);
};

#endif

